# meereceiver.py
# 06.04.2012, N. Ramagnano

import time

class MeeReceiver:

    buffer = ""
    
    def parse(self, string):
        """Parse a string of serial characters.
        Return a list of tuples containing the data
        (   id,
            xacc,
            yacc,
            zacc,
            joystick_up,
            joystick_left,
            joystick_down,
            joystick_right,
            joystick_push,
            button_left,
            button_right)"""
        
        self.buffer += string
        
        list = []
        
        # repeat as long as there is enough data to process
        while len(self.buffer)>=12:
            # check if there is at least one sync word
            if '\xaa' in self.buffer:
                # get the position of the first occurrence
                pos = self.buffer.index('\xaa')
                # proceed only if the whole frame is available
                if len(self.buffer) >= (pos + 12):
                    #~ print "%3d, %f" % (ord(self.buffer[pos+1]),time.time())
                    # check if it is a valid frame
                    if self.fletcher(self.buffer[pos:pos+12]):
                        # interpret frame
                        id = ord(self.buffer[pos+1])
                        xacc_l = ord(self.buffer[pos+2])
                        xacc_h = ord(self.buffer[pos+3])
                        xacc = xacc_l + (xacc_h&1)*256 - 256*(xacc_h&2)
                        yacc_l = ord(self.buffer[pos+4])
                        yacc_h = ord(self.buffer[pos+5])
                        yacc = yacc_l + (yacc_h&1)*256 - 256*(yacc_h&2)
                        zacc_l = ord(self.buffer[pos+6])
                        zacc_h = ord(self.buffer[pos+7])
                        zacc = zacc_l + (zacc_h&1)*256 - 256*(zacc_h&2)
                        joystick_up = ord(self.buffer[pos+8])&0x80 == 0x80
                        joystick_left = ord(self.buffer[pos+8])&0x40 == 0x40
                        joystick_down = ord(self.buffer[pos+8])&0x20 == 0x20
                        joystick_right = ord(self.buffer[pos+8])&0x08 == 0x08
                        joystick_push = ord(self.buffer[pos+8])&0x10 == 0x10
                        button_left = ord(self.buffer[pos+8])&0x02==0x02
                        button_right = ord(self.buffer[pos+8])&0x01==0x01
                        
                        list.append((id,
                                    xacc,
                                    yacc,
                                    zacc,
                                    joystick_up,
                                    joystick_left,
                                    joystick_down,
                                    joystick_right,
                                    joystick_push,
                                    button_left,
                                    button_right))
                    
                    # consume used data
                    self.buffer = self.buffer[pos+12:]
        
        return list
    
    def clear_buffer(self):
        self.buffer = ""
    
    def fletcher(self, frame_string):
        if len(frame_string) == 12:
            pa = 0
            pb = 0
            for n in range(2,10):
                pa = (pa + ord(frame_string[n]))%256
                pb = (pb + pa)%256
            if pa == ord(frame_string[10]) and pb == ord(frame_string[11]):
                return True
        
        return False
    